//////////////////////////////////////////////////////////////////////////////
// SAMPLE: CorBrowser, display all managed processes on the system
//
// Copyright (c) 2002 Vadim Melnik (vmelnik@docsultant.com). 
// All rights reserved.
//
// http://www.docsultant.com/
//

using System;
using System.Collections;
using CorpubProcessLib;
using CORDBLib;

namespace Docsultant.Samples.Interop.CorBrowser
{
	/// <summary>
	/// CorPubHelper - enumerates managed processes and domains
	/// via CLR Publishing interface
	/// </summary>
	public class CorPubHelper
	{
		public delegate bool EnumProcessesProc(ICorPublishProcess proc, object param);
		public delegate bool EnumAppDomainProc(ICorPublishAppDomain domain, object param);


		public static void EnumProcesses(EnumProcessesProc enumProc, COR_PUB_ENUMPROCESS procType, object param)
		{
			ICorPublish cp = new CorpubPublishClass();
			ICorPublishProcessEnum penum;

			// enum managed processes
			cp.EnumProcesses(procType, out penum);

			uint celt;
			penum.GetCount(out celt);

			while( celt-- > 0 )
			{
				ICorPublishProcess proc;
				uint fetch;
				penum.Next(1, out proc, out fetch);
				if( proc!=null )
					if( !enumProc(proc, param) )
						break;
			}
		}

		public static void EnumManagedProcesses(EnumProcessesProc enumProc, object param)
		{
			EnumProcesses(enumProc, COR_PUB_ENUMPROCESS.COR_PUB_MANAGEDONLY, param);
		}

		public static void EnumAppDomains(ICorPublishProcess proc, EnumAppDomainProc enumProc, object param)
		{
			ICorPublishAppDomainEnum penum;
			proc.EnumAppDomains(out penum);

			uint celt;
			penum.GetCount(out celt);

			while( celt-- > 0 )
			{
				ICorPublishAppDomain domain;
				uint fetch;
				penum.Next(1, out domain, out fetch);
				if( domain!=null )
					if( !enumProc(domain, param) )
						break;
			}
		}

		public delegate void GetNameDelegator(uint cchName, out uint pcchName, out ushort szName);

		public static unsafe string GetName(GetNameDelegator f)
		{
			// retrieve process name
			uint cbName = 1024;
			char[] achName = new char[cbName];

			fixed( char* ppchName = &achName[0] )
			{
				f((uint)achName.Length, out cbName, out *(ushort*)ppchName);
				return cbName>1 ? new String(achName, 0, (int)--cbName):"";
			}
		}
	}
};